<?php

namespace App\Http\Controllers;

use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;
use App\Models\ImageDetails;
use App\Models\Users;
use App\Models\Investment;
use App\Models\Setting;
use Illuminate\Http\Request;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use ZipArchive;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Http;
class ApiController extends BaseController
{
    public function saveImageDetails(Request $request)
    {
        try{
            $sriLankaTime  = Carbon::now('Asia/Colombo');

            // $path = $request->file('IMAGE')->store('public/images');
            $path = $request->file('IMAGE')->store('images', 'public');

            // Return the URL of the stored image
            $url = Storage::url($path);
            $serverUrl='http://127.0.0.1/chat_app_backend/public';
            ImageDetails::create([
                'USER_ID' => $request->USER_ID,
                'NAME' => $request->NAME,
                'IS_SEEN' =>0,
                'IMAGE' => $serverUrl.$url,
                'INVESTMENT' =>$request->INVESTMENT,
                'EX' =>$request->EX,
                'CREATED_DATE' =>$sriLankaTime->format('Y-m-d H:i:s')

            ]);
            return true;
        } catch (\Exception $e) {
            return $e;

        }
    }
    public function createUser(Request $request)
    {
        try{
            $user=Users::where("USER_NAME",$request->contact)->first();
        if($user){
            return false;
        }else{
            Users::create([
                'USER_NAME' => $request->contact,
                'CONTACT_NO' => $request->contact,
                'PASSWORD' => $request->password,
                'NAME' => $request->name,
                'LOCATION' =>$request->location,
                'STATUS' =>1
            ]);
            return true;
        }
        } catch (\Exception $e) {
            return $e;
        }
    }
    public function investSave(Request $request)
    {
        $sriLankaTime  = Carbon::now('Asia/Colombo');
        try{
            $userInvest=Investment::where("DATE",$sriLankaTime->format('Y-m-d'))
            ->where("USER_ID", $request->USER_ID)->first();
        if($userInvest){
            Investment::where("ID",$userInvest->ID)->update([
                'CASH_INVEST' => $request->cashInvest,
                'MORNING_INVEST' => $request->morningInvest,
                'EVENING_INVEST' => $request->eveningInvest,
            ]);

            return true;
        }else{
            Investment::create([
                'USER_ID' => $request->USER_ID,
                'CASH_INVEST' => $request->cashInvest,
                'MORNING_INVEST' => $request->morningInvest,
                'EVENING_INVEST' => $request->eveningInvest,
                'DATE' => $sriLankaTime->format('Y-m-d'),
            ]);
            return true;
        }
        } catch (\Exception $e) {
            return $e;
        }
    }
    public function getTodayInvest(Request $request)
    {
        $sriLankaTime  = Carbon::now('Asia/Colombo');
        try{
            $userInvest=Investment::where("DATE",$sriLankaTime->format('Y-m-d'))
            ->where("USER_ID", $request->USER_ID)->first();
            return response()->json(["rows"=> $userInvest]);
        } catch (\Exception $e) {
            return $e;
        }
    }

    public function updateUser(Request $request)
    {
        try{
            Users::where("USER_ID",$request->updateId)->update([
                'USER_NAME' => $request->contact,
                'CONTACT_NO' => $request->contact,
                'PASSWORD' => $request->password,
                'NAME' => $request->name,
                'LOCATION' =>$request->location,
                'STATUS' =>1
            ]);
            return true;
        } catch (\Exception $e) {
            return $e;
        }
    }
    public function userLogin(Request $request)
    {
        try{
            $users = Users::where("USER_NAME",$request->userName)->where("PASSWORD",$request->password)->first();
            $settings=Setting::where("ID",1)->first();
            if($users){
                return response()->json([ "success"=> true, "rows"=> $users,"settings"=>$settings]);
            }else{
                return response()->json([  "success"=> false]);
            }
        } catch (\Exception $e) {
            return $e;
        }
    }
    public function getImageAllByUser(Request $request)
    {
        try{
            $sriLankaTime  = Carbon::now('Asia/Colombo');
            $rows =  DB::select("select * from images_details where USER_ID='".$request->USER_ID."' and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' order by CREATED_DATE asc");
            return response()->json([  "rows"=> $rows]);
        } catch (\Exception $e) {
            return $e;

        }
    }
    public function getImageSearch(Request $request)
    {
        try{
            $date= Carbon::parse($request->DATE);

            // $rows =  DB::select("select * from images_details where USER_ID='".$request->USER_ID."' and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$date->addDays(1)->format('Y-m-d')."' order by CREATED_DATE asc");
            $rows =  DB::select("select * from images_details where USER_ID='".$request->USER_ID."' and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$date->format('Y-m-d')."' order by CREATED_DATE asc");
            return response()->json([  "rows"=> $rows]);
        } catch (\Exception $e) {
            return $e;

        }
    }
    public function getAllUsers(Request $request)
    {
        try{
            $rows = Users::where('STATUS' ,1)->get();
            return  $rows;

        } catch (\Exception $e) {
            return $e;

        }
    }
    public function imageRemoveById(Request $request)
    {
        try{
              ImageDetails::where('ID' , $request->ID)->delete();
            return response()->json([ "SUCCESS"=>$request->ID]);

        } catch (\Exception $e) {
            return $e;

        }
    }
    public function getAllMessages(Request $request)
    {
        try{
            $data=Setting::where("ID",1)->first();
            $morningEnd=($data->MORNING_HOUR_END*60)+$data->MORNING_MINUTES_END;
            $eveningEnd=($data->EVENING_HOUR_END*60)+$data->EVENIN_MINUTES_END;

            $sriLankaTime  = Carbon::now('Asia/Colombo');
            $rows =  DB::select("select u.USER_ID, u.NAME,u.LOCATION,u.CONTACT_NO,
            (select CASH_INVEST from investment where USER_ID=i.USER_ID and DATE_FORMAT(DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."') as CASH_INVEST,
            (select MORNING_INVEST from investment where USER_ID=i.USER_ID and DATE_FORMAT(DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."') as MORNING_INVEST,
            (select EVENING_INVEST from investment where USER_ID=i.USER_ID and DATE_FORMAT(DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."') as EVENING_INVEST,
            (select TOTAL_INVEST from investment where USER_ID=i.USER_ID and DATE_FORMAT(DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."') as TOTAL_INVEST,
            (select sum(INVESTMENT) from images_details where USER_ID=i.USER_ID and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' and INVESTMENT is not null) as CHIT_TOTAL,
            (select COUNT(USER_ID) from images_details where USER_ID=i.USER_ID and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' and EX=1) as EX_COUNT,
            (select COUNT(USER_ID) from images_details where USER_ID=i.USER_ID and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' and ((DATE_FORMAT(CREATED_DATE, '%H') * 60) +DATE_FORMAT(CREATED_DATE, '%i'))> $morningEnd and EX=0) as EVENING_COUNT,
            (select COUNT(USER_ID) from images_details where USER_ID=i.USER_ID and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' and ((DATE_FORMAT(CREATED_DATE, '%H') * 60) +DATE_FORMAT(CREATED_DATE, '%i'))< $morningEnd and EX=0) as MORNING_COUNT,
            (select COUNT(USER_ID) from images_details where USER_ID=i.USER_ID and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."') as NO_OF_IMAGES,
            (select COUNT(USER_ID) from images_details where USER_ID=i.USER_ID and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' AND IS_SEEN=0) as UNSEEN_IMAGES
             from images_details i join users u on u.USER_ID=i.USER_ID
              where u.IS_SUPER_ADMIN='0' and DATE_FORMAT(i.CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."'
              group by i.USER_ID order by i.CREATED_DATE asc");

            return response()->json([ "rows"=>$rows]);

        } catch (\Exception $e) {
            return response()->json([ "error"=>$e]);

        }
    }
    public function getMessagesByUserId(Request $request)
    {
        try{
            $sriLankaTime  = Carbon::now('Asia/Colombo');
            $rows =  DB::select("select * from images_details where USER_ID='".$request->USER_ID."' and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' order by CREATED_DATE asc");
            ImageDetails::where("USER_ID",$request->USER_ID)->update([
                'IS_SEEN' => 1
            ]);
            $setting=Setting::where("ID",1)->first();
            return response()->json([  "rows"=> $rows,"setting"=>$setting]);
        } catch (\Exception $e) {
            return response()->json([ "error"=>$e]);
        }
    }
    public function saveImages(Request $request)
    {
        $users = Users::where("USER_ID",$request->USER_ID)->first();
        $sriLankaTime  = Carbon::now('Asia/Colombo');
        $data=Setting::where("ID",1)->first();
        $morningEnd=($data->MORNING_HOUR_END*60)+$data->MORNING_MINUTES_END;
        $sql="";
        if($request->TIME=="ALL"){
            $sql=" AND DATE_FORMAT(CREATED_DATE, '%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' ";
        }
        if($request->TIME=="MORNING"){
            $sql=" AND ((DATE_FORMAT(CREATED_DATE, '%H') * 60) +DATE_FORMAT(CREATED_DATE, '%i'))< $morningEnd and EX=0";
        }
        if($request->TIME=="EVENING"){
            $sql=" AND ((DATE_FORMAT(CREATED_DATE, '%H') * 60) +DATE_FORMAT(CREATED_DATE, '%i'))>$morningEnd and EX=0";
        }
        if($request->TIME=="EX"){
            $sql=" and EX=1";
        }
        $base64Images = DB::select("select * from images_details where USER_ID='$request->USER_ID' and DATE_FORMAT(CREATED_DATE,'%Y-%m-%d')='".$sriLankaTime->format('Y-m-d')."' $sql order by CREATED_DATE asc");
        if(!$base64Images)
            return "No images found!!!";

        $imagePaths = [];
        $i=1;


        $tempDir = storage_path('app/'.uniqid('temp_images_', true));
        if (!file_exists($tempDir)) {
            mkdir($tempDir, 0755, true);
        }

        foreach ($base64Images as $base64Image) {
            // Decode base64 image data
            $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $base64Image->IMAGE));
            $parseTime= Carbon::parse($base64Image->CREATED_DATE);

            $timeType="";
            if($base64Image->EX==0 && ((($parseTime->format('H')*60)+$parseTime->format('i'))<$morningEnd)){
                $timeType="_MO";
            }
            if($base64Image->EX==0 && ((($parseTime->format('H')*60)+$parseTime->format('i'))>$morningEnd)){
                $timeType="_EV";
            }
            if($base64Image->EX==1){
                $timeType="_EX";
            }
            // Generate a unique filename for each image
            $filename =  $i.$timeType.'_img_invest('.$base64Image->INVESTMENT.")" . '.jpeg';

            $url = $base64Image->IMAGE;
            $imageContent = Http::get($url)->body();
            file_put_contents($tempDir . '/' . $filename, $imageContent);

            // Store the image path for later use
            $imagePaths[] = public_path('temp/' . $filename);
            $i++;
        }

        $zip = new ZipArchive();
        $zipFileName =$users->NAME.'_'.$sriLankaTime->format('Y_m_d')."_".$request->TIME.'.zip';

        $zipFilePath = storage_path('app/public/' . $zipFileName);
        if ($zip->open($zipFilePath, ZipArchive::CREATE) === TRUE) {
            $files = glob($tempDir . '/*');
            foreach ($files as $file) {
                $zip->addFile($file, basename($file));
            }
            $zip->close();
            foreach ($files as $file) {
                unlink($file);
            }
            rmdir($tempDir);
        } else {
            // Handle error
            return response()->json(['error' => 'Unable to create ZIP file'], 500);
        }
        return response()->download($zipFilePath)->deleteFileAfterSend(true);
    }


    public function settingSave(Request $request)
    {
        try{
            if(Setting::where("ID",1)->first()){
                  Setting::where("ID",1)->update([
                    'MORNING_HOUR_END' => $request->morningTimeEndH,
                    'MORNING_MINUTES_END' => $request->morningTimeEndM,
                    'EVENING_HOUR_END' => $request->eveningTimeEndH,
                    'EVENIN_MINUTES_END' => $request->eveningTimeEndM,
                ]);
            }else{
                Setting::create([
                    'MORNING_HOUR_END' => $request->morningTimeEndH,
                    'MORNING_MINUTES_END' => $request->morningTimeEndM,
                    'EVENING_HOUR_END' => $request->eveningTimeEndH,
                    'EVENIN_MINUTES_END' => $request->eveningTimeEndM,
                ]);
            }
            return true;
        } catch (\Exception $e) {
            return $e;
        }
    }
    public function getSetting(Request $request)
    {
        try{
            $data=Setting::where("ID",1)->first();
            return response()->json(["rows"=> $data]);
        } catch (\Exception $e) {
            return $e;
        }
    }
}